// $Id: config.h,v 1.16 2003/01/15 20:41:54 toast Exp $

#ifndef _CONFIG_H_
#define _CONFIG_H_

#include <string>
#include <list>

#include "strtype"
#include "oomon.h"

class OperDef
{
public:
  std::string Handle, Mask, Passwd;
  int Flags;
  OperDef();
  OperDef(const OperDef & def);
};

class LinkDef
{
public:
  std::string Handle, Mask, Passwd;
  int Flags;
  LinkDef();
  LinkDef(const LinkDef & def);
};

class ConnDef
{
public:
  std::string Handle, Hostname, Passwd;
  int Flags;
  int Port;
  ConnDef();
  ConnDef(const ConnDef & def);
};

class YLine
{
public:
  std::string YClass;
  std::string Description;
  YLine();
  YLine(const YLine & def);
};

struct server {
  std::string Hostname, Password, Channels;
  int Port;
};

typedef std::list<OperDef> OperList;
typedef std::list<LinkDef> LinkList;
typedef std::list<ConnDef> ConnList;
typedef std::list<YLine> YLineList;

class Config
{
private:
  static OperList Opers;
  static int Port;
  static StrList Exceptions;
  static StrList Spoofers;
  static LinkList Links;
  static ConnList Connections;
  static YLineList YLines;
  static std::string Nick, UserName, IRCName, OperNick, OperPass;
  static std::string OurHostName;
  static std::string LogFile;
  static std::string MOTD;
  static struct server Server;
  static unsigned long WGvhost;
  static std::string helpFilename;

  static void AddOper(const std::string & Handle, const std::string & Mask,
	       const std::string & Passwd, const std::string & Flags);
  static void AddException(const std::string & Mask);
  static void AddSpoofer(const std::string & Mask);
  static void AddLink(const std::string & Handle, const std::string & Mask,
	       const std::string & Passwd, const std::string & Flags);
  static void AddConn(const std::string & Handle, const std::string & Hostname,
	       const std::string & Passwd, const std::string & Flags, const int Port);
  static void AddYLine(const std::string & YClass, const std::string & Description);
  static bool SetWGvhost(const std::string &);
public:
  static void Clear();						// Clear everything
  static void Load(const std::string filename = DEFAULT_CFGFILE);	// Load configuration
								//  from file
  static bool Auth(const std::string & Handle, const std::string & UserName,
	const std::string & Password, int & Flags, std::string & NewHandle);
  static std::string GetHostName();
  static std::string GetUserName();
  static std::string GetNick() { return Nick; }
  static std::string GetIRCName() { return IRCName; }
  static std::string GetServerHostName() { return Server.Hostname; }
  static int GetServerPort() { return Server.Port; }
  static std::string GetServerPassword() { return Server.Password; }
  static std::string GetOperNick() { return OperNick; }
  static std::string GetOperPass() { return OperPass; }
  static std::string GetChannels() { return Server.Channels; }
  static std::string GetLogFile() { return LogFile; }
  static std::string GetMOTD() { return MOTD; }
  static int GetPort() { return Port; }
  static bool IsOKHost(const std::string &);
  static bool IsOKHost(const std::string &, const std::string &);
  static bool IsOper(const std::string &);
  static bool IsOper(const std::string &, const std::string &);
  static bool IsLinkable(const std::string &);
  static bool GetConn(std::string &, std::string &, int &, std::string &, int &);
  static bool AuthBot(const std::string &, const std::string &, const std::string &);
  static std::string GetYLineDescription(const std::string &);
  static unsigned long GetWGvhost();
  static bool IsSpoofer(const std::string &);
  static std::string getHelpFilename() { return helpFilename; }
};

// User flags
#define UF_NONE			0x0000
#define UF_REMOTE		0x0001
#define UF_MASTER		0x0002
#define UF_GLINE		0x0004
#define UF_KLINE		0x0008
#define UF_DLINE		0x0010
#define UF_OPER			0x0020
#define UF_WALLOPS		0x0040
#define UF_CHANOP		0x0080
#define UF_ALL			0xFFFF

// Link flags
#define LF_NONE			0x0000

// Connect flags
#define CF_NONE			0x0000

#endif
