// $Id: dcc.h,v 1.27 2003/01/14 16:38:28 toast Exp $

#ifndef _DCC_H_
#define _DCC_H_

#include <sys/types.h>
#include <unistd.h>

#include <string>
#include <list>

#include "strtype"
#include "oomon.h"
#include "config.h"

class DCC
{
private:
  static class DCC *items[];
  static int count;

  bool listening, connected, Authed, echoMyChatter;
  int plug;
  std::string UserHost, Nick, Handle;
  int Flags;
  time_t LastActive;

  static void Write(const std::string & text, const DCC *exception);
  static DCC *FindHandle(const std::string &);

  int Parse(std::string text);
  void Chat(const std::string & text);
  void MOTD();
  int PrivConnect(const unsigned long address, const int port,
	const std::string & nick, const std::string & userhost);
  int PrivListen(const std::string & nick, const std::string & userhost);
  int PrivProcess(const fd_set & readset, const fd_set & writeset);
  void PrivSetFD(fd_set & readset, fd_set & writeset) const;
  void Kline(const int Minutes, const std::string & Target,
    const std::string & Reason);
  void KClone(const int Minutes, const std::string & Target);
  void KFlood(const int Minutes, const std::string & Target);
  void KBot(const int Minutes, const std::string & Target);
  void KSpam(const int Minutes, const std::string & Target);
  void KLink(const int Minutes, const std::string & Target);
  void KTrace(const int Minutes, const std::string & Target);
  void KMotd(const int Minutes, const std::string & Target);
  void KInfo(const int Minutes, const std::string & Target);
  void KWingate(const int Minutes, const std::string & Target);
  void KProxy(const int Minutes, const std::string & Target);
  void KPerm(const std::string & Target, const std::string & Reason);
  void Unkline(const std::string & Target);
  void Dline(const std::string & Target, const std::string & Reason);
  void Undline(const std::string & Target);
  void Gline(const int Minutes, const std::string & Target,
    const std::string & Reason);
  void Ungline(const std::string & Target);
  void Help(std::string);
  void Status(const std::string &);
  void TrapCommand(std::string);
public:
  static int Connect(const unsigned long address, const int port,
	  const std::string & nick, const std::string & userhost);
  static int Listen(const std::string & nick, const std::string & userhost);
  static void SetFD(fd_set & readset, fd_set & writeset);
  static int Process(const fd_set & readset, const fd_set & writeset);
  static void Status(const std::string &, const std::string &,
    const DCC *exception = (DCC *) 0);
  static void Status(const std::string &, const std::string &,
    const std::string &, const DCC *exception = (DCC *) 0);
  static bool Chat(const std::string &, const std::string &, std::string,
	  const DCC *exception = (DCC *) 0);
  static void Who(StrList &);
  static void StatsP(StrList &);
  static void WriteByFlags(const std::string & text, const int flags);

  DCC();
  ~DCC();
  int Disconnect();
  int Write(const std::string & msg, const int flags = UF_NONE);
  int Write(StrList & msg, const std::string & prefix = "",
    const std::string & suffix = "\n");
  void NoAccess();
  void NotRemote();
  std::string GetUserHost() const { return UserHost; };
  std::string GetHandle() const { return Handle; };
  int GetIdle() const;
  std::string GetFlags() const;
  bool IsOper() const;
  bool IsConnected() const;
};

// Duh
#define DCC_UNKNOWN	0

// Generic commands
#define DCC_AUTH	1
#define DCC_QUIT	2
#define DCC_CHAT	3
#define DCC_WHO		4
#define DCC_LINKS	5
#define DCC_HELP	6
#define DCC_WATCH	7
#define DCC_MOTD	8
#define DCC_ECHO	9

// +K commands
#define DCC_KLINE	10
#define DCC_UNKLINE	11
#define DCC_KCLONE	12
#define DCC_KFLOOD	13
#define DCC_KBOT	14
#define DCC_KLINK	15
#define DCC_KSPAM	16
#define DCC_KPERM	17
#define DCC_KTRACE	18
#define DCC_KMOTD	19
#define DCC_KINFO	20
#define DCC_KWINGATE	21
#define DCC_KPROXY	22

// +G commands
#define DCC_GLINE	110
#define DCC_UNGLINE	111
#define DCC_GCLONE	112
#define DCC_GFLOOD	113
#define DCC_GBOT	114
#define DCC_GLINK	115
#define DCC_GSPAM	116
#define DCC_GPERM	117
#define DCC_GTRACE	118
#define DCC_GMOTD	119
#define DCC_GINFO	120
#define DCC_GWINGATE	121
#define DCC_GPROXY	122

// +D commands
#define DCC_DLINE	30
#define DCC_UNDLINE	31

// +O commands
#define DCC_KILL	40
#define DCC_KILLLIST	41
#define DCC_MULTI	42
#define DCC_NFIND	43
#define DCC_TRACE	44
#define DCC_CLONES	45
#define DCC_LIST	46
#define DCC_FINDK	47
#define DCC_RELOAD	48
#define DCC_DOMAINS	49
#define DCC_CLASS	50
#define DCC_HMULTI	51
#define DCC_UMULTI	52
#define DCC_VMULTI	53
#define DCC_IPLIST	54
#define DCC_FINDD	55

// +M commands
#define DCC_DIE		60
#define DCC_CONN	61
#define DCC_DISCONN	62
#define DCC_AUTOPILOT	63
#define DCC_RAW		64
#define DCC_TRAP	65

#define DCC_TEST	69

// +C commands
#define DCC_OP		70
#define DCC_JOIN	71
#define DCC_PART	72

#endif
