// $Id: help.cc,v 1.4 2002/11/12 21:18:36 toast Exp $

#include <fstream>
#include <string>
#include <list>
#include <algorithm>

#include "help.h"
#include "config.h"
#include "util.h"


std::list<HelpTopic> Help::topics;
StrList Help::topicList;


bool
Help::haveTopic(const std::string & topic)
{
  for (std::list<HelpTopic>::iterator pos = topics.begin();
    pos != topics.end(); ++pos)
  {
    if (pos->isTopic(topic))
    {
      return true;
    }
  }

  return false;
}


StrList
Help::getIndex(void)
{
  StrList	result;
  std::string	row = std::string("");
  char 		line[MAX_BUFF];
  char		*topic;

  if (topicList.size() <= 0)
  {
    ifstream helpFile(Config::getHelpFilename().c_str());
    while (helpFile.getline(line, sizeof(line) - 1))
    {
      if ((line[0] != '.') && (line[0] != '#'))
      {
        topic = strtok(line, ".\r\n");
        while (topic != (char *) NULL)
        {
	  if (*topic && (topicList.end() == find(topicList.begin(),
	    topicList.end(), std::string(topic))))
	  {
	    if ((topicList.size() == 0) ||
	      (std::string(topic) > topicList.back()))
	    {
	      topicList.push_back(DownCase(std::string(topic)));
	    }
	    else
	    {
	      StrList::iterator pos;
	      for (pos = topicList.begin();
		(pos != topicList.end()) && (*pos < std::string(topic)); ++pos);
              if (pos != topicList.end())
	      {
	        topicList.insert(pos, DownCase(std::string(topic)));
	      }
	    }
	  }
	  topic = strtok((char *) NULL, ".\r\n");
        }
      }
    }
    helpFile.close();
  }

  result.clear();
  result.push_back("\002Help Index:\002");

  for (StrList::iterator pos = topicList.begin();
    pos != topicList.end(); ++pos)
  {
    if (row == std::string(""))
    {
      row = std::string("  ") + DownCase(*pos);
    }
    else
    {
      row += std::string(" ") + DownCase(*pos);
      if (row.length() >= 40)
      {
        result.push_back(row);
        row = std::string("");
      }
    }
  }
  if (row != std::string(""))
  {
    result.push_back(row);
  }

  return result;
}


StrList
Help::getHelp(const std::string & topic)
{
  if (topic == std::string(""))
  {
    return getIndex();
  }
  else if (haveTopic(topic))
  {
    std::list<HelpTopic>::iterator pos;
    for (pos = topics.begin(); (pos != topics.end()) && (!pos->isTopic(topic));
      ++pos);
    if (pos != topics.end())
    {
      return pos->getHelp();
    }
    else
    {
      StrList result;
      result.push_back(std::string("No help available for: ") + topic);
      return result;
    }
  }
  else
  {
    HelpTopic newTopic;
    StrList result;

    result = newTopic.getHelp(topic);

    if (result.size() > 0)
    {
      topics.push_back(newTopic);
    }
    else
    {
      if (newTopic.hadError())
      {
        result.push_back(std::string("Error reading help file!"));
        result.push_back(
	  std::string("Try http://toast.blackened.com/oomon/ for help."));
      }
      else
      {
        result.push_back(std::string("No help available for: ") + topic);
      }
    }
    return result;
  }
}


void
Help::flush(void)
{
  topics.clear();
  topicList.clear();
}
