// $Id: helptopic.cc,v 1.4 2002/11/12 21:18:36 toast Exp $

#include <fstream>
#include <string>
#include <list>
#include <algorithm>

#include "helptopic.h"
#include "config.h"
#include "util.h"

HelpTopic::HelpTopic(void)
{
  topics.clear();
  syntax.clear();
  description.clear();
  example.clear();
  helpLink.clear();
  flags = UF_NONE;
  error = 0;
}


HelpTopic::HelpTopic(const HelpTopic & copy)
{
  topics = copy.topics;
  syntax = copy.syntax;
  description = copy.description;
  example = copy.example;
  helpLink = copy.helpLink;
  flags = copy.flags;
  error = 0;
}


HelpTopic::~HelpTopic(void)
{
}


bool
HelpTopic::isTopic(const std::string & topic)
{
  for (StrList::iterator pos = topics.begin();
    pos != topics.end(); ++pos)
  {
    if (*pos == topic)
    {
      return true;
    }
  }

  return false;
}


StrList
HelpTopic::getHelp(void)
{
  StrList result;

  result.clear();

  if (topics.size() <= 0)
  {
    return result;
  }

  if (syntax.size() > 0)
  {
    result.push_back("\002Syntax:\002");
    for (StrList::iterator pos = syntax.begin();
      pos != syntax.end(); ++pos)
    {
      result.push_back(std::string("  .") + *pos);
    }
  }

  if (description.size() > 0)
  {
    result.push_back("\002Description:\002");
    for (StrList::iterator pos = description.begin();
      pos != description.end(); ++pos)
    {
      result.push_back(std::string("  ") + *pos);
    }
  }

  if (example.size() > 0)
  {
    result.push_back("\002Example:\002");
    for (StrList::iterator pos = example.begin();
      pos != example.end(); ++pos)
    {
      result.push_back(std::string("  ") + *pos);
    }
  }

  result.push_back("\002Required Flags:\002");
  if (flags == UF_NONE)
  {
    result.push_back("  None");
  }
  else
  {
    if (flags & UF_CHANOP)
    {
      result.push_back("  C - Chanop");
    }
    if (flags & UF_DLINE)
    {
      result.push_back("  D - Dline");
    }
    if (flags & UF_GLINE)
    {
      result.push_back("  G - Gline");
    }
    if (flags & UF_KLINE)
    {
      result.push_back("  K - Kline");
    }
    if (flags & UF_MASTER)
    {
      result.push_back("  M - Master");
    }
    if (flags & UF_OPER)
    {
      result.push_back("  O - Oper");
    }
    if (flags & UF_REMOTE)
    {
      result.push_back("  R - Remote");
    }
    if (flags & UF_WALLOPS)
    {
      result.push_back("  W - Wallops");
    }
  }

  if (helpLink.size() > 0)
  {
    std::string row = std::string("");

    result.push_back("\002See Also:\002");
    for (StrList::iterator pos = helpLink.begin();
      pos != helpLink.end(); ++pos)
    {
      if (row == std::string(""))
      {
	row = DownCase(*pos);
      }
      else
      {
	row += std::string(" ") + *pos;
	if (row.length() > 40)
	{
          result.push_back(std::string("  ") + row);
	  row = std::string("");
	}
      }
    }
    if (row != std::string(""))
    {
      result.push_back(std::string("  ") + row);
    }
  }

  return result;
}


StrList
HelpTopic::getHelp(const std::string & topic)
{
  char 		line[MAX_BUFF];
  char		*crlf, *code, *value;
  bool		inTopic = false;

  ifstream	helpFile(Config::getHelpFilename().c_str());
  if (!helpFile.is_open())
  {
    error = true;
    return StrList();
  }

  while (helpFile.getline(line, sizeof(line) - 1))
  {
    crlf = strpbrk(line, "\r\n");
    if (crlf)
    {
      *crlf = (char) 0;
    }

    if (line[0] == '.')
    {
      if (!inTopic)
      {
	continue;
      }

      code = line + 1;
      value = index(code, '.');
      if (value)
      {
	*(value++) = (char) 0;

	if (code[0] == 's')
	{
	  syntax.push_back(value);
	}

	if (code[0] == 'd')
	{
	  description.push_back(value);
	}

	if (code[0] == 'e')
	{
	  example.push_back(value);
	}

	if (code[0] == 'f')
	{
	  if (index(value, 'c'))
	  {
	    flags |= UF_CHANOP;
	  }
	  if (index(value, 'd'))
	  {
	    flags |= UF_DLINE;
	  }
	  if (index(value, 'g'))
	  {
	    flags |= UF_GLINE;
	  }
	  if (index(value, 'k'))
	  {
	    flags |= UF_KLINE;
	  }
	  if (index(value, 'm'))
	  {
	    flags |= UF_MASTER;
	  }
	  if (index(value, 'o'))
	  {
	    flags |= UF_OPER;
	  }
	  if (index(value, 'r'))
	  {
	    flags |= UF_REMOTE;
	  }
	  if (index(value, 'w'))
	  {
	    flags |= UF_WALLOPS;
	  }
	}

	if (code[0] == 'l')
	{
	  helpLink.push_back(value);
	}
      }
    }
    else if (line[0] != '#')
    {
      char *item;

      if (topics.size() > 0)
      {
	break;
      }

      topics.clear();
      item = strtok(line, ".");
      while (item != (char *) NULL)
      {
	topics.push_back(DownCase(item));
	item = strtok(NULL, ".");
      }
      inTopic = (topics.end() != find(topics.begin(), topics.end(),
	DownCase(topic)));
      if (!inTopic)
      {
	topics.clear();
      }
    }
  }
  helpFile.close();

  return getHelp();
}
