// $Id: klines.cc,v 1.11 2003/01/14 22:50:26 toast Exp $

// C++ headers
#include <string>
#include <list>

#include "strtype"
#include "oomon.h"
#include "klines.h"
#include "util.h"
#include "main.h"
#include "log.h"

void KlineList::Clear()
{
  Klines.clear();
}

void KlineList::Add(const std::string & userhost, const std::string & reason)
{
  if (userhost != "")
  {
    #ifdef KLINES_DEBUG
      cout << std::string("Adding ") + lineType + "-line: " << userhost <<
	" (" << reason << ')' << endl;
    #endif
    Klines.push_back(KlineItem(userhost, reason));
  }
}

void KlineList::Find(StrList & Output, const std::string & userhost)
{
  int matches = 0;
  Output.clear();
  Output.push_back(std::string("Finding ") + lineType + "-lines matching " +
    userhost + ':');
  for (std::list<KlineItem>::iterator pos = Klines.begin();
    pos != Klines.end(); ++pos)
  {
    if (MatchesMask(pos->userhost, userhost))
    {
      Output.push_back("  " + pos->userhost + " (" + pos->reason + ')');
      matches++;
    }
  }
  Output.push_back(std::string("End of FIND") + lineType + " " + userhost +
    " (" + IntToStr(Klines.size()) + " " + lineType + "-lines, " + IntToStr(matches) +
    " matches)");
}

void KlineList::Remove(const std::string & userhost)
{
  bool GotMatch;

  do {
    GotMatch = false;
    for (std::list<KlineItem>::iterator pos = Klines.begin();
      pos != Klines.end(); ++pos)
    {
      if (Same(pos->userhost, userhost))
      {
	#ifdef KLINE_DEBUG
	  cout << std::string("Removing ") + lineType + "-line: " <<
	    pos->userhost << endl;
	#endif
	GotMatch = true;
	Klines.erase(pos);
	break;
      }
    }
  } while (GotMatch);
}

// LarzMON added K-Line for [*khknkfg@*.sprint.ca] [Clones are prohibited [Toast@LarzMON]]
void KlineList::ParseAndAdd(std::string Text)
{
  std::string who = FirstWord(Text); // kline adder
  FirstWord(Text); // "added"
  FirstWord(Text); // "K-Line"
  FirstWord(Text); // "for"
  // This won't work all the time, since a badly entered kline might
  // contain spaces. *shrug*
  std::string UserHost = FirstWord(Text);
  std::string Reason = Text;

  // Remove brackets
  UserHost.erase((std::string::size_type) 0, 1);
  UserHost.erase(UserHost.length() - 1, 1);
  Reason.erase((std::string::size_type) 0, 1);
  Reason.erase(Reason.length() - 1, 1);

  std::string msg = getNick(who) + " added " + lineType + "-Line: " +
    UserHost + " (" + Reason + ")";
  Log::Write(msg);
  ::Status(msg, SEND_OPERS_ONLY);

  Add(UserHost, Reason);
}

// Toast has removed the K-Line for: [*@*.monkeys.org] (1 matches)
void KlineList::ParseAndRemove(std::string Text)
{
  std::string who = FirstWord(Text); // kline adder
  FirstWord(Text); // "has"
  FirstWord(Text); // "removed"
  FirstWord(Text); // "the"
  FirstWord(Text); // "K-Line"
  FirstWord(Text); // "for:"
  // Again, spaces in the kline mask will screw this up.
  std::string UserHost = FirstWord(Text);

  // Remove brackets
  UserHost.erase((std::string::size_type) 0, 1);
  UserHost.erase(UserHost.length() - 1, 1);

  std::string msg = getNick(who) + " removed " + lineType + "-Line: " +
    UserHost;
  Log::Write(msg);
  ::Status(msg, SEND_OPERS_ONLY);

  Remove(UserHost);
}

