// Objected Oriented Monitor Bot
// OOMon v1 by Timothy L. Jensen (aka Toast)
// February 1998

// $Id: main.cc,v 1.24 2003/01/16 20:50:02 toast Exp $

// C headers
#include <sys/types.h>
#include <sys/time.h>
#include <signal.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <errno.h>

// C++ headers
#include "oomon.h"
#include <iostream>
#include <fstream>
#include <string>
#include <list>
#include "config.h"
#include "irc.h"
#include "engine.h"
#include "log.h"
#include "proxy.h"
#include "util.h"
#include "main.h"

void Status(const std::string & Notice, const int flags = SEND_ALL_USERS)
{
  DCC::WriteByFlags(Notice + "\n", flags);
  //Remote::Status("*", Notice);
}

void gracefuldie(int sig)
{
  if (sig == SIGTERM) {
    ::Status("Caught SIGTERM -- OOMon terminating");
    Log::Write("Caught SIGTERM -- OOMon stopped");
    IRC::Disconnect();
  } else if (sig == SIGINT) {
    ::Status("Caught SIGINT -- OOMon terminating");
    Log::Write("Caught SIGINT -- OOMon stopped");
    IRC::Disconnect();
  } else {
    abort();
    IRC::Disconnect("Uh-oh!");
  }

  Log::Stop();

  exit(1);
}

void reload()
{
  Config::Clear();
  Config::Load();
  Log::Stop();
  Log::Start();
  Remote::Listen();
}

void ReloadConfig(const std::string & From)
{
  ::Status("Reload CONFIG requested by " + From, SEND_OPERS_ONLY);
  reload();
}

void hangup(int sig)
{
  if (sig != SIGHUP)
    return;
  Log::Write("Caught SIGHUP -- Reloading config");
  ::Status("*** Caught SIGHUP.", SEND_OPERS_ONLY);
  reload();
  signal(SIGHUP,hangup);
}

void pipevec(int sig)
{
  if (sig == SIGPIPE) {
    cerr << "Caught SIGPIPE" << endl;
    signal(SIGPIPE,pipevec);
  }
}

int process()
{
  int select_result;
  struct timeval server_time_out;
  fd_set readfds, writefds;

  for (;;) {
    FD_ZERO(&readfds);
    FD_ZERO(&writefds);
    IRC::SetFD(readfds);
    DCC::SetFD(readfds, writefds);
    Remote::SetFD(readfds);
    Proxy::SetFD(readfds, writefds);

    server_time_out.tv_sec = SERVER_TIME_OUT;
    server_time_out.tv_usec = 0L;

    if ((select_result = select(FD_SETSIZE, &readfds,
		&writefds, (fd_set *) NULL,
		&server_time_out)) > 0) {
      if (IRC::Process(readfds) != 0) {
	IRC::Disconnect();
	IRC::Connect();
      }
      DCC::Process(readfds, writefds);
      Remote::Process(readfds);
      Proxy::Process(readfds, writefds);
    }
#ifdef SERVICES
    Check_Services();
#endif
  }
}

// MOTD()
//
// Writes the Message Of The Day
//
void MOTD(StrList & Output)
{
  ifstream	motdfile;
  char		line[MAX_BUFF];

  motdfile.open(Config::GetMOTD().c_str());
  if (!motdfile.fail())
  {
    Output.push_back("Message Of The Day:");
    while (motdfile.getline(line, MAX_BUFF-1)) {
      Output.push_back(std::string(line));
    }
    Output.push_back("End of MOTD");
  }
  else
  {
    Output.push_back("No MOTD");
  }
  motdfile.close();
}

int main(int argc, char **argv)
{
  ofstream PIDfile;
  pid_t	forkresult;

#ifdef FORK_OOMON
  if (forkresult = fork())
  {
    if (forkresult == -1)
    {
      cerr << "Unable to fork! Error " << IntToStr(errno) << endl;
      return 1;
    }
    return 0;
  }
#endif /* FORK_OOMON */

  PIDfile.open(DEFAULT_PIDFILE);
  PIDfile << getpid();
  PIDfile.close();

  srand(time(NULL));
  signal(SIGSEGV,gracefuldie);
  signal(SIGBUS,gracefuldie);
  signal(SIGTERM,gracefuldie);
  signal(SIGINT,gracefuldie);
  signal(SIGHUP,hangup);
  signal(SIGPIPE,pipevec);

  Config::Load();
  Log::Start();
  Log::Write("OOMon started");

  Remote::Listen();

  if (IRC::Connect() != INVALID) {

    while (!process());

    IRC::Disconnect();
  }

  Log::Write("OOMon stopped");
  Log::Stop();

  return 0;
}
