// $Id: socks4.cc,v 1.1 2003/01/16 20:50:02 toast Exp $

// Std C headers
#include <sys/types.h> 				// for getpwuid(3)
#include <pwd.h>				// for getpwuid(3)
#include <errno.h>				// for errno

// Std C++ headers
#include <iostream>
#include <fstream>
#include <string>

// OOMon headers
#include "oomon.h"
#include "socks4.h"
#include "log.h"
#include "main.h"
#include "engine.h"
#include "irc.h"


Socks4::Socks4(const int fd, const std::string & address,
  const std::string & hostname, const std::string & nick,
  const std::string & userhost)
{
  this->initialize(fd, address, hostname, nick, userhost);
}


// OnConnect()
//
//
bool Socks4::OnConnect()
{
#ifdef PROXY_DEBUG
  cout << "Socks4 detector connected to " << GetAddress() << endl;
#endif

  struct passwd *tmppw = getpwuid(getuid());

  char *tmp;

  if (tmppw)
    tmp = tmppw->pw_name;
  else
    tmp = "unknown";

  unsigned short port = htons(Config::GetServerPort());
  unsigned long dst = IRC::GetHostAddr();

  char buff[2048];

  buff[0] = 4;
  buff[1] = 1;
  strncpy(&buff[2], (char *)&port, 2);
  strncpy(&buff[4], (char *)&dst, 4);
  strncpy(&buff[8], tmp, 1000);

  int m = strlen(tmp) + 5;

  //n = write(plug, buff, m);
  int n = this->Write(buff, m);

  if (n < 0)
  {
    cerr << "Error: Write " << errno << " (" << strerror(errno) << ")" << endl;
    return false;
  }
  else if (n == 0)
  {
    return false;
  }
#ifdef PROXY_DEBUG
  cout << "Socks4 << " << buff << endl;
#endif
  return true;
}


bool Socks4::Parse(std::string text)
{
#ifdef PROXY_DEBUG
  cout << "Socks4 returned: " << int(text[1]) << endl;
#endif

  if (text[1] == 90)
  {
#ifdef PROXY_DEBUG
    cout << "PROXY DETECTED!" << endl;
#endif
    std::string notice = std::string("Socks4 detected for ") + GetNick() +
      +"!" + GetUserhost() + " [" + GetAddress() + "]";
    ::Status(notice, SEND_OPERS_ONLY);
    Log::Write(notice);

#if defined(AUTO_KLINE_PROXIES) || defined(AUTO_DLINE_PROXIES)
    if (GetAutoPilot())
    {
# ifdef AUTO_DLINE_PROXIES
      IRC::Dline("Auto-Dline", GetAddress(), REASON_PROXY);
# else
#  ifdef KLINE_IP_SUPPORTED
      IRC::Kline("Auto-Kline", PROXY_KLINE_TIME, "*@" + GetAddress(),
        REASON_PROXY);
#  else
      IRC::Kline("Auto-Kline", PROXY_KLINE_TIME, "*@" + GetHostname(),
        REASON_PROXY);
#  endif
# endif
    }
    else
#endif
    {
      ::Status(::Make_Kline("*@" + GetHostname(), MK_PROXY,
        PROXY_KLINE_TIME), SEND_KLINERS_ONLY);
    }
  }
  return true;
}

